@@ -1,30 +1,27 @@
 import re
 import unicodedata
-from datetime import datetime
 from difflib import SequenceMatcher
 
 import requests
-
 # from reversion import revisions as reversion
 import reversion
-from apis_core.apis_entities.serializers_generic import EntitySerializer
-from apis_core.apis_labels.models import Label
-from apis_core.apis_vocabularies.models import CollectionType, LabelType, TextType
-
-# from helper_functions.highlighter import highlight_text
-from apis_core.default_settings.NER_settings import autocomp_settings
+from django.apps import apps
 from django.conf import settings
 from django.contrib.auth.models import Group
 from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import ObjectDoesNotExist
 from django.db import models
 from django.db.models.query import QuerySet
-from django.db.models.signals import m2m_changed, post_save
-from django.dispatch import receiver
-from django.urls import NoReverseMatch, reverse
+from django.urls import reverse
 from django.utils.functional import cached_property
 from model_utils.managers import InheritanceManager
 
-from .validators import date_validator
+from apis_core.apis_entities.serializers_generic import EntitySerializer
+from apis_core.apis_labels.models import Label
+from apis_core.apis_vocabularies.models import CollectionType, LabelType, TextType
+# from helper_functions.highlighter import highlight_text
+from apis_core.default_settings.NER_settings import autocomp_settings
+from apis_core.helper_functions import DateParser
 
 NEXT_PREV = getattr(settings, "APIS_NEXT_PREV", True)
 
@@ -50,22 +47,22 @@ class TempEntityClass(models.Model):
         help_text="Should be set to True, if the data record holds up quality standards.",
     )
     start_date = models.DateField(blank=True, null=True)
+    start_start_date = models.DateField(blank=True, null=True)
+    start_end_date = models.DateField(blank=True, null=True)
     end_date = models.DateField(blank=True, null=True)
+    end_start_date = models.DateField(blank=True, null=True)
+    end_end_date = models.DateField(blank=True, null=True)
     start_date_written = models.CharField(
         max_length=255,
         blank=True,
         null=True,
-        validators=[date_validator],
         verbose_name="Start",
-        help_text="Please enter a date (DD).(MM).YYYY",
     )
     end_date_written = models.CharField(
         max_length=255,
         blank=True,
         null=True,
-        validators=[date_validator],
         verbose_name="End",
-        help_text="Please enter a date (DD).(MM).YYYY",
     )
     text = models.ManyToManyField("Text", blank=True)
     collection = models.ManyToManyField("Collection")
@@ -75,16 +72,18 @@ class TempEntityClass(models.Model):
     )
     references = models.TextField(blank=True, null=True)
     notes = models.TextField(blank=True, null=True)
+    published = models.BooleanField(default=False)
     objects = models.Manager()
     objects_inheritance = InheritanceManager()
 
+
     def __str__(self):
         if self.name != "" and hasattr(
             self, "first_name"
         ):  # relation usually don´t have names
-            return "{}, {}".format(self.name, self.first_name)
+            return "{}, {} (ID: {})".format(self.name, self.first_name, self.id)
         elif self.name != "":
-            return self.name
+            return "{} (ID: {})".format(self.name, self.id)
         else:
             return "(ID: {})".format(self.id)
 
@@ -92,56 +91,53 @@ class TempEntityClass(models.Model):
         """Adaption of the save() method of the class to automatically parse string-dates into date objects
         """
 
-        def match_date(date):
-            """Function to parse string-dates into python date objects.
-            """
-            date = date.strip()
-            date = date.replace("-", ".")
-            if re.match(r"[0-9]{4}$", date):
-                dr = datetime.strptime(date, "%Y")
-                dr2 = date
-            elif re.match(r"[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{4}$", date):
-                dr = datetime.strptime(date, "%d.%m.%Y")
-                dr2 = date
-            elif re.match(r"[0-9]{4}\.\.\.$", date):
-                dr = datetime.strptime(date, "%Y...")
-                dr2 = re.match(r"([0-9]{4})\.\.\.$", date).group(1)
-            elif re.match(r"[0-9]{4}\.[0-9]{1,2}\.\.$", date):
-                dr = datetime.strptime(date, "%Y.%m..")
-                dr2 = re.match(r"([0-9]{4})\.([0-9]{1,2})\.\.$", date).group(2)
-                +"." + re.match(r"([0-9]{4})\.([0-9]{1,2})\.\.$", date).group(1)
-            elif re.match(r"[0-9]{4}\.[0-9]{1,2}\.[0-9]{1,2}$", date):
-                dr = datetime.strptime(date, "%Y.%m.%d")
-                dr3 = re.match(r"([0-9]{4})\.([0-9]{1,2})\.([0-9]{1,2})$", date)
-                dr2 = dr3.group(3) + "." + dr3.group(2) + "." + dr3.group(1)
-            elif re.match(r"^\s*$", date):
-                dr = None
-                dr2 = None
-            else:
-                dr = None
-                dr2 = date
-            return dr, dr2
-
         if parse_dates:
+
+            # overwrite every field with None as default
+            start_date = None
+            start_start_date = None
+            start_end_date = None
+            end_date = None
+            end_start_date = None
+            end_end_date = None
+
             if self.start_date_written:
-                self.start_date, self.start_date_written = match_date(
-                    self.start_date_written
-                )
-            else:
-                self.start_date = self.start_date_written = None
+                # If some textual user input of a start date is there, then parse it
+
+                start_date, start_start_date, start_end_date = \
+                    DateParser.parse_date(self.start_date_written)
+
             if self.end_date_written:
-                self.end_date, self.end_date_written = match_date(self.end_date_written)
-            else:
-                self.end_date = self.end_date_written = None
-            if self.name:
-                self.name = unicodedata.normalize("NFC", self.name)
-            super(TempEntityClass, self).save(*args, **kwargs)
-        else:
-            if self.name:
-                self.name = unicodedata.normalize("NFC", self.name)
-            super(TempEntityClass, self).save(*args, **kwargs)
+                # If some textual user input of an end date is there, then parse it
+
+                end_date, end_start_date, end_end_date = \
+                    DateParser.parse_date(self.end_date_written)
+
+            self.start_date = start_date
+            self.start_start_date = start_start_date
+            self.start_end_date = start_end_date
+            self.end_date = end_date
+            self.end_start_date = end_start_date
+            self.end_end_date = end_end_date
+
+        if self.name:
+            self.name = unicodedata.normalize("NFC", self.name)
+
+        super(TempEntityClass, self).save(*args, **kwargs)
+
         return self
 
+    def get_child_entity(self):
+        for x in [x for x in apps.all_models['apis_entities'].values()]:
+            if x.__name__ in list(settings.APIS_ENTITIES.keys()):
+                try:
+                    my_ent = x.objects.get(id=self.id)
+                    return my_ent
+                    break
+                except ObjectDoesNotExist:
+                    pass
+        return None
+
     @classmethod
     def get_listview_url(self):
         entity = self.__name__.lower()
@@ -177,6 +173,13 @@ class TempEntityClass(models.Model):
         else:
             return None
 
+    def get_child_class(self):
+        child = self.get_child_entity()
+        if child:
+            return "{}".format(child.__class__.__name__)
+        else:
+            return "{}".format(child.__class__.__name__)
+
     def get_absolute_url(self):
         entity = self.__class__.__name__.lower()
         if entity == "institution" or len(entity) < 10:
@@ -184,6 +187,8 @@ class TempEntityClass(models.Model):
                 "apis_core:apis_entities:generic_entities_detail_view",
                 kwargs={"entity": entity, "pk": self.id},
             )
+        elif entity == "tempentityclass":
+            return self.get_child_entity().get_absolute_url()
         else:
             return reverse(
                 "apis_core:apis_relations:generic_relations_detail_view",
@@ -260,15 +265,21 @@ class TempEntityClass(models.Model):
             e_b = type(ent).__name__
             if e_a != e_b:
                 continue
-            print(e_b)
-            print(str(ent))
             lt, created = LabelType.objects.get_or_create(name="Legacy name (merge)")
-            l_uri, created = LabelType.objects.get_or_create(name="Legacy URI (merge)")
+            col_list = list(self.collection.all())
+            for col2 in ent.collection.all():
+                if col2 not in col_list:
+                    self.collection.add(col2)
+            for f in ent._meta.local_many_to_many:
+                if not f.name.endswith('_set'):
+                    sl = list(getattr(self, f.name).all())
+                    for s in getattr(ent, f.name).all():
+                        if s not in sl:
+                            getattr(self, f.name).add(s)
             Label.objects.create(label=str(ent), label_type=lt, temp_entity=self)
             for u in Uri.objects.filter(entity=ent):
-                Label.objects.create(
-                    label=str(u.uri), label_type=l_uri, temp_entity=self
-                )
+                u.entity = self
+                u.save()
             for l in Label.objects.filter(temp_entity=ent):
                 l.temp_entity = self
                 l.save()
@@ -327,9 +338,24 @@ class Collection(models.Model):
     parent_class = models.ForeignKey(
         "self", blank=True, null=True, on_delete=models.CASCADE
     )
+    published = models.BooleanField(default=False)
+
+    @classmethod
+    def from_db(cls, db, field_names, values):
+        instance = super().from_db(db, field_names, values)
+        instance._loaded_values = dict(zip(field_names, values))
+        return instance
 
     def __str__(self):
         return self.name
+    
+    def save(self, *args, **kwargs):
+        if hasattr(self, '_loaded_values'):
+            if self.published != self._loaded_values['published']:
+                for ent in self.tempentityclass_set.all():
+                    ent.published = self.published
+                    ent.save()
+        super().save(*args, **kwargs)
 
 
 @reversion.register()
@@ -389,10 +415,29 @@ class Uri(models.Model):
             "relation_pk": self.pk,
             "relation_type": "uri",
             "related_entity": self.entity.name,
+            "related_entity_url": self.entity.get_absolute_url(),
+            "related_entity_class_name": self.entity.__class__.__name__.lower(),
             "uri": self.uri,
         }
         return result
 
+    @classmethod
+    def get_listview_url(self):
+        return reverse("apis_core:apis_metainfo:uri_browse")
+
+    @classmethod
+    def get_createview_url(self):
+        return reverse('apis_core:apis_metainfo:uri_create')
+
+    def get_absolute_url(self):
+        return reverse('apis_core:apis_metainfo:uri_detail', kwargs={'pk': self.id})
+
+    def get_delete_url(self):
+        return reverse('apis_core:apis_metainfo:uri_delete', kwargs={'pk': self.id})
+
+    def get_edit_url(self):
+        return reverse('apis_core:apis_metainfo:uri_edit', kwargs={'pk': self.id})
+
 
 @reversion.register()
 class UriCandidate(models.Model):
@@ -415,7 +460,6 @@ class UriCandidate(models.Model):
         for endp in autocomp_settings[cn.title()]:
             url = re.sub(r"/[a-z]+$", "/entity", endp["url"])
             params = {"id": self.uri}
-            print(url, params)
             res = requests.get(url, params=params, headers=headers)
             if res.status_code == 200:
                 if endp["fields"]["descr"][0] in res.json()["representation"].keys():
@@ -430,7 +474,9 @@ class UriCandidate(models.Model):
                 return (label, desc)
 
 
-@receiver(post_save, sender=Uri, dispatch_uid="remove_default_uri")
-def remove_default_uri(sender, instance, **kwargs):
-    if Uri.objects.filter(entity=instance.entity).count() > 1:
-        Uri.objects.filter(entity=instance.entity, domain="apis default").delete()
+#@receiver(post_save, sender=Uri, dispatch_uid="remove_default_uri")
+#def remove_default_uri(sender, instance, **kwargs):
+#    if Uri.objects.filter(entity=instance.entity).count() > 1:
+#        Uri.objects.filter(entity=instance.entity, domain="apis default").delete()
+
+
