@@ -447,7 +447,7 @@ class RDFParser(object):
         self._preserve_uri_minutes = preserve_uri_minutes
         self._uri_check = uri_check
         self._use_preferred = use_preferred
-        self.objct = ContentType.objects.get(app_label=app_label_entities, model=kind).model_class()
+        self.objct = ContentType.objects.get(app_label=app_label_entities, model__iexact=kind).model_class()
         self._app_label_relations = app_label_relations
         self.kind = kind
         self._foreign_keys = []
